#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gdk/gdkkeysyms.h>
#include <time.h>
#include "kf.h"
#include "jabber.h"
#include "gui.h"
#include "chat.h"

static GHashTable *kf_chat_windows = NULL;

void kf_chat_init (void);
static void kf_chat_window_real (KfChat *chat);
KfChat *kf_chat_new (const gchar *jid);
gboolean on_my_textview_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void kf_chat_send_msg (KfChat *chat);
static void kf_chat_display (KfChat *chat, KfJabberMessage *msg, gboolean moja);
gboolean on_chat_eventbox_realize (GtkWidget *widget, GdkEvent *event, gpointer user_data);
gboolean on_chat_eventbox_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
gboolean    on_tab_eventbox_button_press_event (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data);
void on_chat_menu_add_activate (GtkMenuItem *menuitem, gpointer user_data);
void on_chat_menu_message_activate (GtkMenuItem *menuitem, gpointer user_data);
void on_chat_menu_clear_activate (GtkMenuItem *menuitem, gpointer user_data);
void on_chat_menu_info_activate (GtkMenuItem *menuitem, gpointer user_data);
gboolean on_chat_window_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);

void on_chat_tabify_activate     (GtkMenuItem *menuitem,
                                            gpointer user_data);
static void kf_chat_tabify (KfChat *chat);
void on_chat_windowify_activate     (GtkMenuItem *menuitem,
                                            gpointer user_data);
static void kf_chat_windowify (KfChat *chat);
gboolean    on_kf_chat_focused             (GtkWidget *widget,
                                            GdkEventFocus *event,
                                            gpointer user_data);
static void kf_chat_update (KfChat *chat);

void kf_chat_init (void) {
	if (kf_chat_windows == NULL) {
		kf_chat_windows = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
	}
}

/* Shows a KfJabberMessage */
void kf_chat_show (KfJabberMessage *msg) {
	KfChat *chat;
	
//	kf_chat_init (); 
	if ((chat = g_hash_table_lookup (kf_chat_windows, msg->from)) == NULL) {
		chat = kf_chat_new (msg->from);
		g_hash_table_insert (kf_chat_windows, g_strdup (chat->jid), chat);
	}
		
	kf_chat_display (chat, msg, FALSE);
	chat->unread++;

	if (chat->mode == KF_CHAT_WINDOW && kf_preferences_get_int ("autoPopup")) {
		gtk_widget_show (chat->window);
		gdk_window_raise ((chat->window)->window);
	}
	
	kf_chat_update (chat);
}

/* Displays a chat window for *jid, creates a new one if necessary */
void kf_chat_window (const gchar *jid) {
	KfChat *chat;

	kf_chat_init (); 
	if ((chat = g_hash_table_lookup (kf_chat_windows, jid)) == NULL) {
		chat = kf_chat_new (jid);
		g_hash_table_insert (kf_chat_windows, g_strdup (jid), chat);
	}

	kf_chat_window_real (chat);
}

/* Internal - displays a chat */
static void kf_chat_window_real (KfChat *chat) {
		
	if (chat->mode == KF_CHAT_WINDOW) {
		if (chat->window) {
			gtk_container_add (GTK_CONTAINER (chat->container), chat->content);

			gtk_widget_show (chat->window);
			gdk_window_raise ((chat->window)->window);
		} else {
			chat->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
			gtk_container_add (GTK_CONTAINER (chat->window), chat->content);

			gtk_widget_show (chat->window);
			gdk_window_raise ((chat->window)->window);
		}
	} else if (chat->mode == KF_CHAT_TAB) {
		extern GladeXML *iface;
		GtkWidget *notebook;
		gint page;

		notebook = glade_xml_get_widget (iface, "roster_notebook");
		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), chat->content);

		if (page == -1) {
			GtkWidget *label;
			GtkWidget *eventbox;
//			GtkStyle *sn;
//			GtkStyle *se;

			eventbox = gtk_event_box_new ();
//			eventbox = gtk_hbox_new (FALSE, 2);
			g_signal_connect (G_OBJECT (eventbox), 
		                      "button_press_event",
                		      G_CALLBACK (on_tab_eventbox_button_press_event),
		                      chat);
			label = gtk_label_new (chat->jid);
			gtk_container_add (GTK_CONTAINER (eventbox), label);
//			gtk_box_pack_start (GTK_BOX (eventbox), label, FALSE, FALSE, 0);
			gtk_widget_show (label);

//			se = eventbox->style;
//			sn = notebook->style;

//			se->bg[GTK_STATE_PRELIGHT].red = 0xffff;
			//sn->bg[GTK_STATE_NORMAL];
//
			chat->label = label;
			gtk_notebook_append_page (GTK_NOTEBOOK (notebook), chat->content, eventbox);
		} else {
			gtk_widget_show (chat->content);
			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), page);
		}
	}
	kf_chat_update (chat);
}

/* Creates a new KfChat - a structure representing a chat window */
KfChat *kf_chat_new (const gchar *jid) {
	KfChat *chat;
	GtkWidget *textview, *scrolledwindow, *menu, *eventbox;
	GtkTextTag *my_text, *his_text;
	GtkLabel *label;
	gchar *cropped_jid, *str, *titel;
	const gchar *title;
	KfJabberRosterItem *item;

	cropped_jid = g_strdup (jid);
	for (str = cropped_jid; *str != '\0' && *str != '/'; str++); /* Usunicie zasobu */
	*str = '\0';
	item = kf_jabber_roster_item_get (cropped_jid);
	g_free (cropped_jid);

	chat = g_malloc (sizeof (KfChat));
	chat->item = item;
	chat->unread = 0;
	chat->jid = g_strdup (jid);
	chat->glade = glade_xml_new (kf_gui_find_file ("chat.glade"), NULL, NULL); 
	glade_xml_signal_autoconnect (chat->glade);
	chat->content = gtk_widget_ref (glade_xml_get_widget (chat->glade, "vbox1"));
//	chat->window = NULL;
	chat->window = glade_xml_get_widget (chat->glade, "chat_window");
	chat->container = glade_xml_get_widget (chat->glade, "frame1");
	chat->label = NULL;
	textview = glade_xml_get_widget (chat->glade, "his_textview");
	chat->view = GTK_TEXT_VIEW (textview);
	chat->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
	textview = glade_xml_get_widget (chat->glade, "my_textview");
	chat->my_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
	g_object_set_data (G_OBJECT (textview), "Chat", chat);
	g_object_set_data (G_OBJECT (chat->content), "Chat", chat);
	scrolledwindow = glade_xml_get_widget (chat->glade, "scrolledwindow1");
	chat->adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolledwindow));
	chat->status_image = GTK_IMAGE (glade_xml_get_widget (chat->glade, "status_image"));

	menu = glade_xml_get_widget (chat->glade, "chat_menu");
	g_object_set_data (G_OBJECT (menu), "KfChat", chat);

	menu = glade_xml_get_widget (chat->glade, "tab_menu");
	g_object_set_data (G_OBJECT (menu), "KfChat", chat);

	label = GTK_LABEL (glade_xml_get_widget (chat->glade, "from_label"));
	gtk_label_set_text (label, jid);

	chat->my_text = gtk_text_buffer_create_tag (chat->buffer, "my_text",
			"pixels-above-lines", 4, "foreground", "navy", "weight", "bold", NULL);
	chat->his_text = gtk_text_buffer_create_tag (chat->buffer, "his_text",
			"pixels-above-lines", 4, "foreground", "red", "weight", "bold", NULL);
	
	if (kf_preferences_get_int ("tabDefault")) {
		chat->mode = KF_CHAT_TAB;
		kf_chat_tabify (chat);
	} else {
		chat->mode = KF_CHAT_WINDOW;
	}
	
	if (item) {
		gtk_image_set_from_pixbuf (chat->status_image,
				kf_gui_pixmap_get_status (item->presence));
	}

	eventbox = glade_xml_get_widget (chat->glade, "chat_eventbox");
	if (eventbox) {
		g_signal_connect (G_OBJECT (eventbox), 
                      "button_press_event",
                      G_CALLBACK (on_chat_eventbox_button_press_event),
                      chat);
	}

	return chat;
}

/* Wstawianie \n przez Shift+Enter i wysyanie przez Enter */
gboolean    on_my_textview_key_press_event (GtkWidget *widget,
                                            GdkEventKey *event,
                                            gpointer user_data)
{
	if (event->keyval == GDK_Return) {
		KfChat *chat;

		/* Ugly, but works */
		if (event->state == 021)
			return FALSE;

		chat = g_object_get_data (G_OBJECT (widget), "Chat");
		if (chat == NULL)
			g_error ("Nie ma czata!\n");
		kf_chat_send_msg (chat); 
		return TRUE;
	}
	return FALSE;
}

static void kf_chat_send_msg (KfChat *chat) {
	GtkTextIter start, end;
	gchar *body;
	KfJabberMessage *msg;	/* Wiadomo */
	

	gtk_text_buffer_get_bounds (chat->my_buffer, &start, &end);
	body = gtk_text_buffer_get_text (chat->my_buffer, &start, &end, FALSE);
	gtk_text_buffer_delete (chat->my_buffer, &start, &end);

	/* Kreacja wiadomoci */
	msg = kf_jabber_message_new ();
	msg->to = g_strdup (chat->jid);
	msg->body = body;
	msg->type = KF_JABBER_MESSAGE_TYPE_CHAT;

	kf_jabber_message_send (msg);
	kf_chat_display (chat, msg, TRUE);
//	kf_jabber_message_free (msg);
}

static void kf_chat_display (KfChat *chat, KfJabberMessage *msg, gboolean moja) {
	GtkTextIter iter;
	gchar *caption;
	struct tm *time;
	gint offset = 0;
	
	time = localtime ((time_t *) &(msg->stamp));

	if (strstr (msg->body, "/me ") == msg->body) {
		caption = g_strdup_printf ("[%02d:%02d] * %s ", time->tm_hour, time->tm_min,
			moja?"Ja":((chat->item)?chat->item->display_name:msg->from));
		offset = 4;
	} else {
		caption = g_strdup_printf ("[%02d:%02d] %s:\n", time->tm_hour, time->tm_min,
			moja?"Ja":((chat->item)?chat->item->display_name:msg->from)); 
	}

	gtk_text_buffer_get_end_iter (chat->buffer, &iter);

	gtk_text_buffer_insert_with_tags (chat->buffer, &iter, caption, -1,
			moja?(chat->my_text):(chat->his_text), NULL);
	
	if (kf_preferences_get_int ("useEmoticons")) {
		kf_emoticons_insert_text (chat->buffer, msg->body + offset);
	} else {
		gtk_text_buffer_insert (chat->buffer, &iter, msg->body + offset, -1);	
	}
	gtk_text_buffer_get_end_iter (chat->buffer, &iter);
	gtk_text_buffer_insert (chat->buffer, &iter, "\n", 1);

	if (chat->adj->value == chat->adj->upper - chat->adj->page_size)
		gtk_text_view_scroll_to_mark (chat->view, 
			gtk_text_buffer_get_insert (chat->buffer) , 0.0, TRUE, 0.0, 1.0);
	
	g_free (caption);

	/* Message is displayed, so don't need it anymore */
//	kf_jabber_message_free (msg);
}

/*
 * Ustawia ikonk obecnoci w okienku czata
 */
void kf_chat_update_status (const gchar *jid, KfJabberPresenceType type, const gchar *status) {
	KfChat *chat;

	if ((chat = g_hash_table_lookup (kf_chat_windows, jid))) {
		gtk_image_set_from_pixbuf (chat->status_image, kf_gui_pixmap_get_status (type));
		return;
	}
}

gboolean    on_chat_eventbox_button_press_event (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data) {

	GtkWidget *menu;
	KfChat *chat;

	chat = (KfChat *) user_data;
	if (chat == NULL)
		g_error ("Nie ma chata!\n");

	menu = glade_xml_get_widget (chat->glade, "chat_menu");
	if (menu == NULL)
		g_error ("Nie ma menu!\n");
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
}
gboolean    on_tab_eventbox_button_press_event (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data) {

	GtkWidget *menu;
	KfChat *chat;

	if (event->button == 3) {
		chat = (KfChat *) user_data;
		if (chat == NULL)
			g_error ("Nie ma chata!\n");

		menu = glade_xml_get_widget (chat->glade, "tab_menu");
		if (menu == NULL)
			g_error ("Nie ma menu!\n");
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
		return TRUE;
	} else {
		return FALSE;
	}
}


void        on_chat_menu_add_activate      (GtkMenuItem *menuitem,
                                            gpointer user_data)
{	
	KfChat *chat;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));

	chat = g_object_get_data (G_OBJECT (menu), "KfChat");
	if (chat) {
		GtkWidget *win, *jid_entry;
		extern GladeXML *iface;

		win = glade_xml_get_widget (iface, "add_contact_window");
		jid_entry = glade_xml_get_widget (iface, "add_jid");
		gtk_entry_set_text (GTK_ENTRY (jid_entry), chat->jid);
		kf_gui_show (win);
	}
}

void        on_chat_menu_message_activate  (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
	KfChat *chat;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));

	chat = g_object_get_data (G_OBJECT (menu), "KfChat");
	if (chat) 
		kf_new_message (chat->jid);
}

void        on_chat_menu_clear_activate    (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
	KfChat *chat;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));

	chat = g_object_get_data (G_OBJECT (menu), "KfChat");

	if (chat) {
		GtkTextIter start, end;
		gtk_text_buffer_get_bounds (chat->buffer, &start, &end);
		gtk_text_buffer_delete (chat->buffer, &start, &end);
	}
}

void        on_chat_menu_info_activate     (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
	KfChat *chat;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));

	chat = g_object_get_data (G_OBJECT (menu), "KfChat");
	if (chat) 
		kf_vcard_get (chat->jid);
}

gboolean	on_chat_window_key_press_event      (GtkWidget *widget,
                                            GdkEventKey *event,
                                            gpointer user_data) {

	/* ESC zamyka okienko */
	if (event->keyval == GDK_Escape) {
		gtk_widget_hide (widget);
	}
	return FALSE;
}

void on_chat_tabify_activate     (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
	KfChat *chat;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));

	chat = g_object_get_data (G_OBJECT (menu), "KfChat");
	if (chat) 
		kf_chat_tabify (chat);
}

/* Moves a chat window to a tab */
static void kf_chat_tabify (KfChat *chat) {
	GtkWidget *container;
	chat->mode = KF_CHAT_TAB;

	container = gtk_widget_get_parent (chat->content);
	gtk_widget_ref (chat->content);
	gtk_container_remove (GTK_CONTAINER (container), chat->content);
	kf_chat_window_real (chat);
	gtk_widget_unref (chat->content);

	gtk_widget_hide (chat->window);
}

void on_chat_windowify_activate     (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
	KfChat *chat;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));

	chat = g_object_get_data (G_OBJECT (menu), "KfChat");
	if (chat) 
		kf_chat_windowify (chat);
}


void on_close_tab1_activate                (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
//	extern GladeXML *iface;
	KfChat *chat;
//	GtkWidget *notebook;
	GtkWidget *menu;

	menu = gtk_widget_get_parent (GTK_WIDGET (menuitem));
	chat = g_object_get_data (G_OBJECT (menu), "KfChat");

	gtk_widget_hide (chat->content);
//	notebook = glade_xml_get_widget (iface, "roster_notebook");
//	page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), chat->content);
	
//	gtk_widget_ref (chat->content);
//	gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
//	if (chat) 
//		kf_chat_windowify (chat);
}



/* Moves a chat window into a window */
static void kf_chat_windowify (KfChat *chat) {
	extern GladeXML *iface;
	gint page;
	GtkWidget *notebook;
	GtkWidget *container;
	chat->mode = KF_CHAT_WINDOW;

	notebook = glade_xml_get_widget (iface, "roster_notebook");
	page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), chat->content);
	
	gtk_widget_ref (chat->content);
	gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
	chat->label = NULL;
	kf_chat_window_real (chat);
	gtk_widget_unref (chat->content);
}

/* Event - chat window is focused */
gboolean    on_kf_chat_focused             (GtkWidget *widget,
                                            GdkEventFocus *event,
                                            gpointer user_data)
{
	GladeXML *glade;
	GtkWidget *content;
	KfChat *chat;
	
	glade = glade_get_widget_tree (widget);
	content = glade_xml_get_widget (glade, "vbox1");
	chat = g_object_get_data (G_OBJECT (content), "Chat");
	g_assert (chat);
	chat->unread = 0;
	kf_chat_update (chat);
}

/* Updates chat title/tab label */
static void kf_chat_update (KfChat *chat) {
	const gchar *nick;

	nick = chat->item?chat->item->display_name:chat->jid;
	if (chat->mode == KF_CHAT_WINDOW) {
		gchar *text;

		if (chat->unread)
			text = g_strdup_printf (_("* Chat with '%s' (%d)"), nick, chat->unread);
		else
			text = g_strdup_printf (_("Chat with '%s'"), nick, chat->unread);

		gtk_window_set_title (GTK_WINDOW (chat->window), text);
		g_free (text);
	} else {
		extern GladeXML *iface;
		GtkWidget *notebook;
		gint page;

		notebook = glade_xml_get_widget (iface, "roster_notebook");
		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), chat->content);
		if (page != -1) {
			GtkWidget *label;
			gchar *text;
			GdkColor color = {0, 0xFFFF, 0, 0};
			GdkColor black = {0, 0, 0, 0};

//			label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (notebook), chat->content);
			label = chat->label;
			g_assert (label);
			if (chat->unread) {
				text = g_strdup_printf (_("%s (%d)"), nick, chat->unread);
				gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &color);
			} else {
				text = g_strdup_printf (_("%s"), nick, chat->unread);
				gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &black);
			}
			gtk_label_set_text (GTK_LABEL (label), text);
			g_free (text);
		}
	}
}

void        on_roster_notebook_switch_page                  (GtkNotebook *notebook,
                                            GtkNotebookPage *page,
                                            guint page_num,
                                            gpointer user_data)
{
	GladeXML *glade;
	GtkWidget *content;
	KfChat *chat;
	
	if (page_num != 0) {
		content = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page_num);
		chat = g_object_get_data (G_OBJECT (content), "Chat");
		g_assert (chat);
		chat->unread = 0;
		kf_chat_update (chat);
	}
}


