#include <gtk/gtk.h>
#include <glade/glade.h>
#include "preferences.h"

static void        kf_settings_toggle             (GtkToggleButton *togglebutton,
                                            gpointer user_data);
static void        kf_settings_changed     (GtkEditable *editable,
                                            gpointer user_data);
static void kf_settings_connect (GladeXML *glade, const gchar *wname, gchar *option);
static void kf_settings_connect_entry (GladeXML *glade, const gchar *wname, gchar *option);

/*
 * Callback dla pstryczka w okienku konfiguracji
 */
static void        kf_settings_toggle             (GtkToggleButton *togglebutton,
                                            gpointer user_data) {
	kf_preferences_set_int ((const gchar *) user_data, gtk_toggle_button_get_active (togglebutton));
}

/*
 * Callback dla GtkEntry w okienku konfiguracji
 */
static void        kf_settings_changed     (GtkEditable *editable,
                                            gpointer user_data) {
	const gchar *text;

	text = gtk_entry_get_text (GTK_ENTRY (editable));
	
	kf_preferences_set ((const gchar *) user_data, text);
}


void kf_settings (void) {
	GtkWidget *win;
	extern GladeXML *iface;
	static gint x = 0;

	win = glade_xml_get_widget (iface, "preferences");

	if (x == 0) {
		kf_settings_connect (iface, "checkbutton3", "colourRows");
		kf_settings_connect (iface, "checkbutton4", "useEmoticons");
		kf_settings_connect (iface, "radiobutton2", "messageDefault");
		kf_settings_connect (iface, "checkbutton5", "autoPopup");
		kf_settings_connect (iface, "checkbutton6", "soundEnabled");
		kf_settings_connect (iface, "checkbutton7", "tabDefault");
		kf_settings_connect_entry (iface, "preferences_browser", "browser");
		kf_settings_connect_entry (iface, "preferences_player", "soundPlayer");
		x = 1;
	}

	kf_accounts_editor (iface);
	
	kf_gui_show (win);
}

static void kf_settings_connect (GladeXML *glade, const gchar *wname, gchar *option) {
	GtkWidget *widget;

	widget = glade_xml_get_widget (glade, wname);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
			kf_preferences_get_int (option));
	g_signal_connect (GTK_OBJECT (widget), "toggled",
                     G_CALLBACK (kf_settings_toggle), option);
}

static void kf_settings_connect_entry (GladeXML *glade, const gchar *wname, gchar *option) {
	GtkWidget *widget;

	widget = glade_xml_get_widget (glade, wname);
	gtk_entry_set_text (GTK_ENTRY (widget),
			kf_preferences_get_string (option));
	g_signal_connect (GTK_OBJECT (widget), "changed",
                     G_CALLBACK (kf_settings_changed), option);
}


