#include <gtk/gtk.h>
#include <glade/glade.h>
#include <loudmouth/loudmouth.h>
#include "kf.h"
#include "jabber.h"

typedef struct {
	gchar *field1;
	gchar *field2;
	gchar *widget;
} KfVCardField;

static KfVCardField vcard_fields[] = {
	{"fn", NULL, "fn"},
	{"nickname", NULL, "nickname"},
	{"bday",NULL, "bday"},
	{"url",NULL, "www"},
	{"email", NULL, "email"},
	{"org","orgname","orgname"},
	{"org","orgunit","orgunit"},
	{"title",NULL,"orgpos"},
	{"role",NULL,"orgrole"},
	{"tel","voice","phone"},
	{"adr","street","street"},
	{"adr","extadd","street2"},
	{"adr","locality","city"},
	{"adr","region","state"},
	{"adr","pcode","pcode"},
	{"adr","country","country"},
	{NULL,NULL,NULL}};

static void kf_vcard_retrieve (const gchar *jid, GtkWidget *win);
static
LmHandlerResult kf_vcard_hendel             (LmMessageHandler *handler,
                                             LmConnection *connection,
                                             LmMessage *message,
                                             gpointer user_data);

void        on_nick_change_clicked         (GtkButton *button,
                                            gpointer user_data);
static void rename_jid (const gchar *jid, const gchar *name);
static void vcard_display (LmMessageNode *node, GladeXML *glade);
static void setwidget (GladeXML *glade, const gchar *name, const gchar *value);
static void setabout (GladeXML *glade, const gchar *value);
static void setsub (GladeXML *glade, KfJabberRosterItem *item);

void kf_vcard_get (const gchar *jid) {
	KfJabberRosterItem *item;
	GladeXML *glade;
	gchar *jidx = g_strdup (jid);
	GtkWidget *win, *jabber_id;
	const gchar *window_title;
	gchar *title;

	kf_jabber_jid_crop (jidx);
	glade = glade_xml_new (kf_find_file ("vcard.glade"), NULL, NULL);
	win = glade_xml_get_widget (glade, "vcard_window");
	window_title = gtk_window_get_title (GTK_WINDOW (win));
	glade_xml_signal_autoconnect (glade);

	item = kf_jabber_roster_item_get (jidx);
	if (item) {
		GtkWidget *entry;

		entry = glade_xml_get_widget (glade, "vcard_name");
		gtk_entry_set_text (GTK_ENTRY (entry), item->display_name);

		setsub (glade, item);

		title = g_strdup_printf (window_title, item->display_name);
	} else {
		GtkWidget *widget;
		
		widget = glade_xml_get_widget (glade, "vcard_name");
		gtk_widget_set_sensitive (widget, FALSE);
		widget = glade_xml_get_widget (glade, "nick_change");
		gtk_widget_set_sensitive (widget, FALSE);
		
		title = g_strdup_printf (window_title, jidx);
	}

	gtk_window_set_title (GTK_WINDOW (win), title);
	g_free (title);

	jabber_id = glade_xml_get_widget (glade, "jabber_id");
	gtk_label_set_text (GTK_LABEL (jabber_id), jidx);

	g_object_set_data_full (G_OBJECT (win), "jid", jidx, g_free);
	kf_vcard_retrieve (jidx, win);
}

static void kf_vcard_retrieve (const gchar *jid, GtkWidget *window) {
	extern LmConnection *kf_jabber_connection;
	LmMessage *msg;
	LmMessageNode *node;
	LmMessageHandler *h;

	h = lm_message_handler_new (kf_vcard_hendel, window, NULL);
	msg = lm_message_new_with_sub_type (jid, LM_MESSAGE_TYPE_IQ, LM_MESSAGE_SUB_TYPE_GET);
	node = lm_message_node_add_child (msg->node, "vCard", NULL);
	lm_message_node_set_attribute (node, "xmlns", "vcard-temp");

	lm_connection_send_with_reply (kf_jabber_connection, msg, h, NULL);
	lm_message_unref (msg);
}

static
LmHandlerResult kf_vcard_hendel             (LmMessageHandler *handler,
                                             LmConnection *connection,
                                             LmMessage *message,
                                             gpointer user_data) {
	GtkWidget *win = user_data;
	GladeXML *glade = glade_get_widget_tree (win);
	GtkWidget *label = glade_xml_get_widget (glade, "vcard_text");
	LmMessageNode *node;

	gtk_label_set_text (GTK_LABEL (label), lm_message_node_to_string (message->node));
	
	node = message->node->children;
	while (node) {
		if (!g_ascii_strcasecmp (node->name, "vcard")) {
			vcard_display (node->children, glade);
		}

		node = node->next;
	}
}

void        on_nick_change_clicked         (GtkButton *button,
                                            gpointer user_data)
{
	GladeXML *glade;
	GtkWidget *entry, *window;
	const gchar *jid, *name;

	glade = glade_get_widget_tree (GTK_WIDGET (button));
	entry = glade_xml_get_widget (glade, "vcard_name");
	window = glade_xml_get_widget (glade, "vcard_window");
	name = gtk_entry_get_text (GTK_ENTRY (entry));
	jid = g_object_get_data (G_OBJECT (window), "jid");

	rename_jid (jid, name);
}


static void rename_jid (const gchar *jid, const gchar *name) {
	LmMessage *msg;
	LmMessageNode *node;
	extern LmConnection *kf_jabber_connection;

	msg = lm_message_new_with_sub_type (jid, LM_MESSAGE_TYPE_IQ, LM_MESSAGE_SUB_TYPE_SET);
	node = lm_message_node_add_child (msg->node, "query", NULL);
	lm_message_node_set_attribute (node, "xmlns", "jabber:iq:roster");

	node = lm_message_node_add_child (node, "item", NULL);
	lm_message_node_set_attribute (node, "name", name);
	lm_message_node_set_attribute (node, "jid", jid);

	kf_connection_send (NULL, msg, NULL);

	lm_message_unref (msg);
}

static void vcard_display (LmMessageNode *node, GladeXML *glade) {

	while (node) {
		KfVCardField *field;
		if (!g_ascii_strcasecmp (node->name, "desc")) {
			setabout (glade, node->value);
			node = node->next;
			continue;
		}
		for (field = vcard_fields; field->field1; field++) {
			if (!g_ascii_strcasecmp (node->name, field->field1)) {
				if (field->field2 == NULL) {
					setwidget (glade, field->widget, node->value);
					break;
				} else {
					KfVCardField *field2;
					LmMessageNode *child = node->children;

					while (child) {
						if (!g_ascii_strcasecmp (child->name, field->field2)) {
							setwidget (glade, field->widget, child->value);
							break;
						}
						child = child->next;
					}
				}
			}
		}
		
		node = node->next;
	}
	
}

static void setwidget (GladeXML *glade, const gchar *name, const gchar *value) {
	GtkWidget *widget;

	widget = glade_xml_get_widget (glade, name);
	if (widget)
		gtk_entry_set_text (GTK_ENTRY (widget), value);
	else 
		g_print ("Cannot find widget %s\n", name);
}

static void setabout (GladeXML *glade, const gchar *value) {
	GtkWidget *tv;
	GtkTextBuffer *buffer;

	tv = glade_xml_get_widget (glade, "about");
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tv));
	gtk_text_buffer_set_text (buffer, value, -1);
}

static void setsub (GladeXML *glade, KfJabberRosterItem *item) {
	GtkWidget *label;
	static gchar *subs[] = {
		"<b>none</b>",
		"<b>from</b>",
		"<b>to</b>",
		"<b>both</b>",
		"<b>unknown</b>"};
	
		g_return_if_fail (item);
		label = glade_xml_get_widget (glade, "subscription_text");
		gtk_label_set_markup (GTK_LABEL (label), subs[item->subscription]);
}
